/**
* @license
* Copyright © 2015 Intuilab
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Intuilab.
*/

/**
* Inheritance on EventEmitter base class
* @type {EventEmitter}
*/
SimpleCounter.prototype = new EventEmitter();        // Here's where the inheritance occurs
SimpleCounter.prototype.constructor = SimpleCounter;

/**
* @constructor
*/
function SimpleCounter() {
    this.InitialValue = 0;
    this.Count = this.InitialValue;
    this.CoerceValue = false;
    this.MinValue = 0;
    this.MaxValue = 100;
	
	//this.defineEvents(['InitialValueChanged','CoerceValueChanged','MinValueChanged','MaxValueChanged']);
	//this.defineEvents(['InitialValueChanged']);
}

//Setters
SimpleCounter.prototype.setInitialValue = function (InitialValue) {
    if (this.InitialValue != InitialValue) {
        this.InitialValue = InitialValue;
        if (this.CoerceValue)
            this.InitialValue = this._CoerceValue(this.InitialValue);

        this.Count = this.InitialValue;
        this.emit('InitialValueChanged', [this.InitialValue]);
        this.emit('CountChanged', [this.Count]);
    }

};



SimpleCounter.prototype.setCoerceValue = function (CoerceValue) {
    if (this.CoerceValue != CoerceValue)
	{
		this.CoerceValue = CoerceValue;
		this.emit('CoerceValueChanged', [CoerceValue]);
		if (this.CoerceValue) {
			//refresh InitialValue 
			this.InitialValue = this._CoerceValue(this.InitialValue);
			this.emit('InitialValueChanged', [this.InitialValue]);
			
			this.Count = this._CoerceValue(this.Count);
			this.emit('CountChanged', [this.Count]);
		}
	}
}

SimpleCounter.prototype.setMinValue = function (MinValue) {
	if (this.MinValue != MinValue){
		this.MinValue = MinValue;
		this.emit('MinValueChanged', [MinValue]);
		if (this.CoerceValue) {
			//refresh InitialValue 
			this.InitialValue = this._CoerceValue(this.InitialValue);
			this.emit('InitialValueChanged', [this.InitialValue]);
			
			this.Count = this._CoerceValue(this.Count);
			this.emit('CountChanged', [this.Count]);
		}
	}
}


SimpleCounter.prototype.setMaxValue = function (MaxValue) {
	if (this.MaxValue != MaxValue){
		this.MaxValue = MaxValue;
		this.emit('MaxValueChanged', [MaxValue]);
		if (this.CoerceValue) {
			//refresh InitialValue 
			this.InitialValue = this._CoerceValue(this.InitialValue);
			this.emit('InitialValueChanged', [this.InitialValue]);
			
			this.Count = this._CoerceValue(this.Count);
			this.emit('CountChanged', [this.Count]);
		}
	}
}

// Internal method to coerce values between min & max
SimpleCounter.prototype._CoerceValue = function (value) {
	var v = (value > this.MaxValue) ? this.MaxValue : (value < this.MinValue) ? this.MinValue : value;	
	return v;
}


/**
* Add one to the counter
*/
SimpleCounter.prototype.AddOne = function () {
    this.Count++;
    
    //Check if the value is in the defined boundaries
    if (this.CoerceValue) 
        this.Count = this._CoerceValue(this.Count);
    
    this.emit('CountChanged', [this.Count]);
};
/**
* Add the argument value to the counter
* @param increment the increment value
*/
SimpleCounter.prototype.Add = function (increment) {
    this.Count += increment;
    //Check if the value is in the defined boundaries
    if (this.CoerceValue)
        this.Count = this._CoerceValue(this.Count);
    this.emit('CountChanged', [this.Count]);
};
/**
* Remove one to the counter
*/
SimpleCounter.prototype.SubstractOne = function () {
    this.Count--;
    //Check if the value is in the defined boundaries
    if (this.CoerceValue)
        this.Count = this._CoerceValue(this.Count);
    this.emit('CountChanged', [this.Count]);
};
/**
* Remove the argument value to the counter
* @param increment the increment value
*/
SimpleCounter.prototype.Substract = function (increment) {
    this.Count -= increment;
    //Check if the value is in the defined boundaries
    if (this.CoerceValue)
        this.Count = this._CoerceValue(this.Count);
    this.emit('CountChanged', [this.Count]);
};
/**
* Remove one to the counter
*/
SimpleCounter.prototype.Reset = function () {
    this.Count = this.InitialValue;
    this.emit('CountChanged', [this.Count]);
};

/**
* Directly sets a value
* @param set the current value
*/
SimpleCounter.prototype.SetNewCountValue = function (value) {
    this.Count = value;
    //Check if the value is in the defined boundaries
    if (this.CoerceValue)
        this.Count = this._CoerceValue(this.Count);
    this.emit('CountChanged', [this.Count]);
};


